' Lawson Labs, Inc.
' 74 4th Ave. NW
' Kalispell, MT  59901
' 800 321-5355 or 406 257-5355

' Model 202 24-Bit Delta-Sigma Converter            Rev 3
' July 26, 1994                                     microcode version 132

GOTO START
'****  Define program variables                                        ****
SETVARS:
  SOTOKEN% = &H88        'normal sign-on
  SDGROUP = 20           'average for standard deviation
  REDIM SD#(SDGROUP)     'standard deviation array
  SDTOT# = 0             'total for standard deviation calc
  SDCOUNT = 1            'counter for standard deviation
  REDIM BAUD$(5)         'Baud rate strings
  REDIM CC%(5)           'external channel counts for scanning
  REDIM CHAN%(5)         'external channel codes for scanning
  BAUD$(0) = "9600"
  BAUD$(1) = "4800"
  BAUD$(2) = "2400"
  BAUD$(3) = "1200"
  BAUD$(4) = "600"
  BAUD$(5) = "300"
  BAUD% = 0          'Baud rate code can be 0 to 5, default to 9600
  RESERVE% = 0       'reserved for future use
  REDIM SCANTYPE$(2)            'scan type labels
  SCANTYPE$(0) = "Single Channel"
  SCANTYPE$(1) = "Normal"       'index is STYPE%
  SCANTYPE$(2) = "Self-Calibration"
  BITS16% = 0        'word length codes
  BITS24% = 128
  BIPOLAR% = 0       'bipolar/ unipolar codes
  UNIPOLAR% = 16
  MNRATE = 10        'minimum data rate for A/D with 10 MHz crystal
  MAXRATE = 1027     'maximum data rate
  ADCLK = 19531.25   '10000000/512 for 10 MHz crystal
  LASTDIGOUT% = 256  'initalize to invalid code
  FALSE = 0
  TRUE = -1
  GETSI% = TRUE      'flag means "need to enter a Scan Interval"
  POLLED% = TRUE     'flag for polled or scanning mode
  FIRSTTIME% = TRUE
  GOSUB SETM202VARS  'M202 set variables specific to the model 202
RETURN

'**************************************************************************
'***********************  PROGRAMMER'S MODEL  *****************************
'**************************************************************************
'The operating mode, voltage range, data rate and gain are set by sending a
'24-bit mode word to the M202. The mode word is sent as three 8-bit bytes.

' MODEREGHI%    BIT                      VARIABLE
               ' 7   operating mode
               ' 6   operating mode
               ' 5   operating mode
               ' 4   gain code           GAIN%
               ' 3   gain code           GAIN%
               ' 2   gain code           GAIN%
               ' 1   must be zero
               ' 0   standby             STBY%

' MODEREGMID%   BIT                      VARIABLE
               ' 7   24bit/16bit         WL%       (wordlength)
               ' 6   not used
               ' 5   must be zero
               ' 4   uni/bipolar         BU%
               ' 3   not used
               ' 2   rate code bit 10    RATE
               ' 1   rate code bit  9    RATE
               ' 0   rate code bit  8    RATE

' MODEREGLO%    BIT                      VARIABLE
               ' 7   rate code bit  7    RATE
               ' 6   rate code bit  6    RATE
               ' 5   rate code bit  5    RATE
               ' 4   rate code bit  4    RATE
               ' 3   rate code bit  3    RATE
               ' 2   rate code bit  2    RATE
               ' 1   rate code bit  1    RATE
               ' 0   rate code bit  0    RATE

START:
  GOSUB READCONFIG                'start from last set up saved to disk

'****  CHANGES -  BAUD%, COM$, INPBUF%, ECHOFLAG%                      ****
'****  CALLS   -  SIGNON, SENDMRESET, POLLOOP, EXITPROGRAM             ****
COMMENU:
  KEY OFF: CLS
  LOCATE 6, 22: PRINT "SERIAL COMMUNICATION  MENU"
  LOCATE 10, 20: PRINT "1) Baud Rate              "; BAUD$(BAUD%)
  LOCATE 12, 20: PRINT "2) Serial port            "; COM$;
  LOCATE 14, 20: PRINT "3) Input buffer          "; INPBUF%; " bytes"
  LOCATE 16, 20: PRINT "4) Serial check           "
  LOCATE 18, 16: PRINT "Enter) Start";
  LOCATE 20, 18: PRINT "Esc) Exit program";
  A$ = INPUT$(1)
  IF A$ = "1" THEN BAUD% = (BAUD% + 1) MOD 6   'select baud rate
  IF A$ = "2" THEN IF COM$ = "COM1:" THEN COM$ = "COM2:" ELSE COM$ = "COM1:"
  IF A$ = "3" THEN
    LOCATE 25, 1: INPUT "            Buffer size (bytes) 1-32767"; TEMP
    IF (TEMP >= 1) AND (TEMP <= 32767) THEN INPBUF% = TEMP
  END IF
  IF A$ = "4" THEN
    ECHOFLAG% = TRUE: GOSUB SIGNON    'test communications
    GOSUB SENDMRESET: ECHOFLAG% = FALSE
  END IF
  IF A$ = CHR$(13) THEN GOSUB POLLOOP: GOTO COMMENU
  IF A$ = CHR$(27) THEN GOSUB EXITPROGRAM
  GOTO COMMENU

'**************************************************************************
'**************************   SUBROUTINES   *******************************
'**************************************************************************
'****      sign-on establishes communications with the model 202       ****
'****  USES    -  LOC(1), INPBYTE%, BAUD%, ECHOFLAG%                   ****
'****  CHANGES -  DOSYSCAL%, OUTBYTE%, CHECKSUM%, NUMCHARS%, WAITIME   ****
'****  CALLS   -  SET300BAUD, GETCHAR, SETBAUD, ECHOTEST               ****
'****             WAITXX, PRINTERROR, EMPTYBUFFER                      ****
SIGNON:
  DOSYSCAL% = TRUE
  WAKEUPCALLS% = 0     'counter for wakeup bytes sent
  E$ = "": ON ERROR GOTO 1000
WAKEMODE:
  GOSUB SET300BAUD     'start at 300 baud
  OUTBYTE% = &H0       'command to reset the M202
  CLS : LOCATE 10, 30: PRINT "Signing on "
WAKEUP:
  WAKEUPCALLS% = WAKEUPCALLS% + 1                     '50 tries
  IF WAKEUPCALLS% = 50 THEN
     IF E$ = "" THEN E$ = " can't sign on"
     GOTO PRINTERROR
  END IF
  XX = .2: GOSUB WAITXX
  PRINT #1, CHR$(OUTBYTE%);
  XX = .2: GOSUB WAITXX
  GOSUB EMPTYBUFFER                 'otherwise, check for correct response
  IF INPBYTE% <> 3 THEN GOTO WAKEUP
                   '202 sends 3 if ready to sign on
                   'it may take several tries to sign on
                   '202 is now awake and waiting for sign-on token
  XX = .2: GOSUB WAITXX
  CHECKSUM% = 0    'clear running checksum
  CLS
  PRINT #1, CHR$(SOTOKEN%);    'send signon token
  XX = .1: GOSUB WAITXX     'wait .1 second
  PRINT #1, CHR$(BAUD%); 'send baud code
  WAITIME = 1: NUMCHARS% = 1: GOSUB WAITCHAR
  GOSUB GETCHAR
  IF INPBYTE% <> BAUD% THEN E$ = " Bad baud": GOTO WAKEUP
  GOSUB SETBAUD          'set the baud rate
  XX = .1: GOSUB WAITXX                 'wait .1 sec for power supply
  IF ECHOFLAG% THEN GOSUB ECHOTEST    'do a communication check
  OUTBYTE% = &H0                'echo-off token
  PRINT #1, CHR$(OUTBYTE%);   'send echo-off token
  GOSUB SETMODEREG
  OUTB1% = MODEREGHI%: OUTB2% = MODEREGMID%
  GOSUB sendpacket            'send mode register data
  OUTB1% = MODEREGLO%: GOSUB sendpacket
  OUTB1% = AVERAGE%: OUTB2% = FILTER%
  GOSUB sendpacket            'send averaging & filter select  data
  OUTB1% = RESERVE%: IF POLLED% THEN OUTB2% = 1 ELSE OUTB2% = 0
  GOSUB sendpacket            'send polled mode data
READMODE:
  NUMCHARS% = 3: MODEOK% = TRUE
  GOSUB WAITCHAR  'wait for 3 characters
  GOSUB GETCHAR
  IF (INPBYTE% AND &H1F) <> (MODEREGHI% AND &H1F) THEN MODEOK% = FALSE
  GOSUB GETCHAR
  IF INPBYTE% <> MODEREGMID% THEN MODEOK% = FALSE
  GOSUB GETCHAR
  IF INPBYTE% <> MODEREGLO% THEN MODEOK% = FALSE
' beep means the mode was not properly received
  IF MODEOK% = FALSE THEN BEEP: E$ = " Bad mode registers": GOTO WAKEMODE
  'M202 setup counters
  IF FIRSTTIME% THEN GOSUB INITCOUNTERS: FIRSTTIME% = FALSE
  ON ERROR GOTO 0    'turn off error trapping
RETURN

'****  Error handler for signon                                        ****
1000 CLOSE #1
  OPEN COM$ + "300,N,8,1,BIN,RS,CS,DS,CD,RB" + STR$(INPBUF%) FOR RANDOM AS #1
  RESUME NEXT     'ignore serial errors during sign-on
EMPTYBUFFER:
  IF LOC(1) <> 0 THEN GOSUB GETCHAR: GOTO EMPTYBUFFER
RETURN

'****  Set serial port to 300 baud                                     ****
'****  USES    -  COM$, INPBUF%                                        ****
SET300BAUD:
  CLOSE
  OPEN COM$ + "300,N,8,1,BIN,RS,CS,DS,CD,RB" + STR$(INPBUF%) FOR RANDOM AS #1
RETURN

'****  Set serial port to the selected com port & baud, then           ****
'****  calculates the time to wait for a conversion                    ****
'****  USES    -  COM$, BAUD%                                          ****
'****  CALLS   -  CALCWAIT                                             ****
SETBAUD:                        'set serial port to selected baud rate
  CLOSE
  s$ = COM$ + BAUD$(BAUD%) + ",N,8,1,BIN,RS,CS,DS,CD,RB" + STR$(INPBUF%)
  OPEN s$ FOR RANDOM AS #1
  GOSUB CALCWAIT              'set default time to wait
RETURN

'****  Gathers data while in polled mode                               ****
'****  USES    -  DOSYSCAL%, STBY%, BU%, GAIN%, CANCELFLAG%, CHANNEL%  ****
'****             EXTCCODE%
'****  CHANGES -  POLLED%, MAX, MIN                                    ****
'****  CALLS   -  TOGGLESTANDBY, SIGNON, SETCCODE, WAITXX, SYSCAL      ****
'****             GETCONV, VOLTSCALE, SD, CALCBITS, DOCOMMAND          ****
'****             SCANMENU, DOMRESET, VOLTDISPLAY                      ****
POLLOOP:
  POLLED% = TRUE
  IF STBY% = 1 THEN GOSUB TOGGLESTANDBY  'set not standby
  GOSUB SIGNON                           'sign-on
  OUTB1% = &H86: OUTB2% = 0              'find Model 202 version
  GOSUB sendpacket
  NUMCHARS% = 2: GOSUB WAITCHAR          'wait for 2 chars
  GOSUB GETCHAR
  IF INPBYTE% <> &H86 THEN E$ = "version command failure": GOTO PRINTERROR
  GOSUB GETCHAR
  VERSION = INPBYTE%
  GOSUB SETCCODE                       'set the control code
  CLS
  H$ = "    COUNT       VOLTS       MAX-MIN    RESOLUTION    CHANNEL     DAC "
  LASTCSR = 3
POLLEDLOOP:
  IF DOSYSCAL% THEN GOSUB SYSCAL  'calibrate if needed
  IF A$ <> "" THEN
    LOCATE 1, 1: PRINT H$;           'header
    IF BU% = BIPOLAR% THEN PRINT "  +/-";  ELSE PRINT "  +  ";
    PRINT USING "  ###x"; GAIN%;
    LOCATE 25, 1
    PRINT "SPACE=MAX-MIN, M=command Menu, N=scaN, Esc=serial menu";
    LOCATE 2, 1: PRINT SPACE$(80): LOCATE 24, 1: PRINT SPACE$(80);
    A$ = ""
  END IF
  LOCATE LASTCSR, 1
  IF STBY% = 1 THEN PRINT "   In standBy    ", : GOTO KEYSTROKE
  GOSUB GETCONV                      'get an A/D conversion
  IF CANCELFLAG% THEN GOTO KEYSTROKE
  PRINT USING " ##,###,###"; COUNT#; 'display the raw count
  GOSUB VOLTSCALE: GOSUB VOLTDISPLAY 'display voltage
  IF MAX < VOLTS THEN MAX = VOLTS    'update maximum
  IF MIN > VOLTS THEN MIN = VOLTS    'update minimum
  PRINT USING "   ###.######"; MAX - MIN;   'display the difference
  GOSUB SD                           'calculate standard deviation
  GOSUB CALCBITS                            'print rms resolution in bits
  PRINT USING "       ## "; MAXBITS% - i;
  PRINT "       Chan="; CHANNEL%;           'display current channel
  IF DACENABLE% = 8 THEN PRINT "  enablEd  " ELSE PRINT "  disablEd"
  IF CSRLIN = 24 THEN LOCATE 3, 1
  LASTCSR = CSRLIN
'****  Get and process a keystroke if needed                             ****
KEYSTROKE:
  IF A$ = "" THEN A$ = UCASE$(INKEY$) 'if no keystroke, check for one
  GOSUB DOCOMMAND
  IF A$ = "M" THEN GOSUB COMMANDMENU
  IF A$ = "N" THEN GOSUB SCANMENU: CLS : GOTO POLLEDLOOP
  IF A$ = CHR$(27) THEN GOSUB DOMRESET: RETURN
  GOTO POLLEDLOOP

'****   Calculate and display effective resolution in bits             ****
'****   USES    - STDV, TEMPSTDV, I                                    ****
CALCBITS:
  i = 0
  TEMPSTDV = STDV
CALCNEXT:                                    
  IF TEMPSTDV >= 1 THEN i = i + 1: TEMPSTDV = TEMPSTDV / 2: GOTO CALCNEXT
RETURN

'****  Do maximum filter settling time, then return                    ****
'****  USES    - FTIME                                                 ****
'****  CALLS   - WAITXX                                                ****
FDELAY:
  LOCATE 25, 1: PRINT SPACE$(70);
  LOCATE 25, 27: PRINT "Filter delay";
  XX = FTIME * 2 + .5                 'set wait time
  GOSUB WAITXX
RETURN

'****  Perform the appropriate calibration                             ****
'****  USES    - STBY%, CHANNEL%, EXTCCODE%, A$, WORDCNT%, WAITTIME    ****
'****  CHANGES - NUMCHARS%                                             ****
'****  CALLS   - TOGGLESTANDBY, SENDADMODE, CLEARCOMMAND, SENDPACKET   ****
'****            WAITCHAR, GETCHAR                                     ****
DOCAL:                                  'do an A/D calibration
  IF STBY% THEN GOSUB TOGGLESTANDBY: GOSUB SENDADMODE   'out of standby
  GOSUB CLEARCOMMAND
  OUTB2% = (CHANNEL% * 16) + DACENABLE%   'calibrate on current channel
  ORGW = WAITIME: WAITIME = WAITIME + FTIME + 2
  IF A$ = "O" THEN
    OUTB1% = &H82                           'offset cal command
    LOCATE 25, 27: PRINT "Offset Calibration";
    END IF
  IF A$ = "F" THEN
    OUTB1% = &H83                            'fullscale cal command
    LOCATE 25, 27: PRINT "Full scale Calibration";
    END IF
  GOSUB sendpacket
  NUMCHARS% = WORDCNT% + 1: GOSUB WAITCHAR         'wait for chars
  FOR i = 1 TO WORDCNT% + 1: GOSUB GETCHAR: NEXT   'discard them
  WAITIME = ORGW
EXITDOCAL:
RETURN

'****  Do a system calibration, restore the original channel select,   ****
'****  skip a conversion, then flag system calibration not required    ****
'****  USES    - CHANNEL%, GAIN%                                       ****
'****  CHANGES - DOSYSCAL%                                             ****
'****  CALLS   - DOCAL, SENDADMODE, SETCCODE, FDELAY                   ****
SYSCAL:
  ORGGAIN% = GAIN%
  ORGCHAN% = CHANNEL%                'save channel data
  IF GAIN% <> 1 THEN GAIN% = 1: GOSUB SENDADMODE
  CHANNEL% = 7: A$ = "O": GOSUB DOCAL             'do offset calibration
  CHANNEL% = 6: A$ = "F": GOSUB DOCAL             'do fullscale calibration
  IF GAIN% <> ORGGAIN% THEN
    GAIN% = ORGGAIN%: GOSUB SENDADMODE
    CHANNEL% = 7: A$ = "O": GOSUB DOCAL             'do offset calibration
  END IF
  CHANNEL% = ORGCHAN%
  GOSUB SETCCODE: GOSUB FDELAY
  DOSYSCAL% = FALSE                  'reset flag
RETURN

'****  Get a conversion, update running checksum                       ****
'****  USES    - WORDCNT%, WL%, CANCELFLAG%                            ****
'****  CHANGES - COUNT#, COUNT1#, COUNT2#, CHECKSUM%                   ****
'****  CALLS   - SENDPACKET, WAITCHARORKEY, GETCHAR                    ****
GETCONV:
  OUTB1% = &H81                        'get conversion token
  OUTB2% = 0              'select A/D converter
  GOSUB sendpacket
  NUMCHARS% = WORDCNT% + 1: GOSUB WAITCHARORKEY    'wait for 2 or 3 char's
  IF CANCELFLAG% THEN GOTO EXITGETCONV
  GOSUB GETCHAR
  IF INPBYTE% <> &H81 THEN E$ = " Get conv token failure": GOTO PRINTERROR
GCONV:
  GOSUB GETCHAR: BYTE1# = INPBYTE%
  GOSUB GETCHAR: BYTE2# = INPBYTE%
  IF WL% THEN GOSUB GETCHAR: BYTE3# = INPBYTE% ELSE BYTE3# = 0
                            'get 3rd byte if 24-bit mode
  COUNT# = BYTE1# + BYTE2# * 256 + BYTE3# * 65536  'calc raw counts
EXITGETCONV:
RETURN

'****  Calculate standard deviation, STDV                              ****
'****  USES    - SD#, COUNT#, SDCOUNT, SDGROUP                         ****
'****  CHANGES - SDTOT#, STDV, SDCOUNT                                 ****
SD:
  SD#(SDCOUNT) = COUNT#
  SDTOT# = SDTOT# + COUNT#
  IF SDCOUNT <> SDGROUP THEN SDCOUNT = SDCOUNT + 1: GOTO EXITSD
  STDV = 0
  SDTOT# = SDTOT# / SDGROUP
  FOR N = 1 TO SDGROUP: STDV = STDV + ((SDTOT# - SD#(N)) ^ 2): NEXT N
  STDV = SQR(STDV / (SDGROUP - 1))
CLEARSDCOUNT:
  SDTOT# = 0
  SDCOUNT = 1
EXITSD:
RETURN

'**** Calculate the A/D control register values                        ****
'**** USES    - GAIN%, STBY%, WL%, BU%                                 ****
'**** CHANGES - MODEREGHI%, MODEREGMID%                                ****
SETMODEREG:
  GOSUB CALCRATE                        'first set RATE bits
  G% = -1: TEMP% = GAIN%
SELECTGAIN:          
  G% = G% + 1                           'G%=0 for x1, 1 for x2, 2 for x4, etc.
  IF TEMP% > 1 THEN TEMP% = TEMP% * .5: GOTO SELECTGAIN
  MODEREGHI% = (G% * 4) OR STBY%        'set gain & power down bits
  MODEREGMID% = MODEREGMID% OR WL% OR BU% 'set word length, bi/unipolar mode
RETURN

'**** Toggle A/D power down mode                                       ****
'**** CHANGES - STBY%                                                  ****
TOGGLESTANDBY:
  IF STBY% = 0 THEN STBY% = 1 ELSE STBY% = 0  '1 = in standby
RETURN

'****  Scale reading from counts to volts                              ****
'****  USES    - BU%, WL%, COUNT#                                      ****
'****  CHANGES - VOLTS                                                 ****
VOLTSCALE:
  VOLTS = (COUNT# * .0000002980232#)            '24-bit
  IF WL% = BITS16% THEN VOLTS = VOLTS * 256     '16-bit
  IF BU% = BIPOLAR% THEN VOLTS = VOLTS * 2 - 5  'bipolar
RETURN

'****  Display volts                                                   ****
VOLTDISPLAY:
  PRINT USING "   ##.######"; VOLTS;            'display volts
RETURN

'****  Clear maximum & minimum voltage                                 ****
'****  CHANGES - MAX, MIN                                              ****
ZERODELTA:
  MAX = -5: MIN = 5
RETURN

COMMANDMENU:
  CLS
  LOCATE 2, 30: PRINT "COMMAND  MENU"; SPACE$(20); "version="; VERSION
  LOCATE 4, 5: PRINT "C) A/D input Channel     "; CHANNEL%
  LOCATE 6, 5: PRINT "E) DAC                   ";
  IF DACENABLE% = 8 THEN PRINT "enablEd " ELSE PRINT "disablEd"
  LOCATE 8, 5: PRINT "G) Gain setting          "; GAIN%
  LOCATE 10, 5: PRINT "W) Word length           "; MAXBITS%; " bits"
  LOCATE 12, 5: PRINT "R) sample Rate        ";
  PRINT USING "####.#"; RATE; : PRINT "  Hz"
  LOCATE 14, 5
  IF BU% = UNIPOLAR% THEN PRINT "P) uni";  ELSE PRINT ; "P) bi";
  PRINT "Polar mode";
  LOCATE 16, 5: PRINT "A) Averaging factor    "; : PRINT USING "#####"; NUMAVG
  LOCATE 4, 45: PRINT "Z) ";
  PRINT "DAC settings ";
  LOCATE 6, 45: PRINT "D) Digital inputs   ";
  IF A$ = "D" THEN
    PRINT LASTDIGINP%;
    LOCATE 5, 64: PRINT "Decimal  76543210"
    LOCATE 6, 73:
    FOR NN% = 7 TO 0 STEP -1
    IF LASTDIGINP% AND 2 ^ NN% THEN PRINT "1";  ELSE PRINT "0";
    NEXT NN%
  END IF
  LOCATE 8, 45: PRINT "U) digital oUtput   ";
  IF LASTDIGOUT% <> 256 THEN PRINT LASTDIGOUT% 'print only after "U" command
  LOCATE 10, 45: PRINT "K) checKsum         ";
  IF A$ = "K" THEN PRINT PRINTSUM%
  LOCATE 12, 45: PRINT "B) ";
  IF STBY% = 1 THEN PRINT "in standBy mode" ELSE PRINT "out of standBy mode"
  LOCATE 14, 45: PRINT "O) Offset calibration "
  LOCATE 16, 45: PRINT "F) Full scale calibration ";
  LOCATE 18, 45: PRINT "S) System calibration"
  LOCATE 20, 1: PRINT "Enter) Start"
  LOCATE 20, 43: PRINT "Esc) To exit"

  A$ = UCASE$(INPUT$(1))
  IF A$ = CHR$(27) THEN RETURN
  IF A$ = CHR$(13) THEN : CLS : RETURN
  GOSUB DOCOMMAND
  GOTO COMMANDMENU

DOCOMMAND:
  IF A$ = "A" THEN GOSUB NEXTAVG     'select averaging
  IF A$ = "B" THEN GOSUB TOGGLESTANDBY   'toggle A/D power down
  IF A$ = "C" THEN GOSUB CONTROLCODE
  IF A$ = "D" THEN GOSUB GETDIGIN    'read the digital inputs
  IF A$ = "E" THEN GOSUB TOGGLEDACS  'toggle dac enable
  IF A$ = "F" THEN GOSUB DOCAL       'calibrate
  IF A$ = "G" THEN GAIN% = (GAIN% * 2) MOD 255 'select the gain
  IF A$ = "K" THEN GOSUB GETCHKSUM   'display running checksum
  IF A$ = "O" THEN GOSUB DOCAL       'calibrate
  IF A$ = "P" THEN GOSUB SELECTBP    'bipolar or unipolar mode
  IF A$ = "R" THEN GOSUB SELECTRATE  'select the rate
  IF A$ = "S" THEN GOSUB SYSCAL      'system calibration
  IF A$ = "U" THEN GOSUB SETDIGOUT   'set the digital outputs
  IF A$ = "W" THEN GOSUB SELECTWL    'select the word length
  IF A$ = "Z" THEN GOSUB ANALOGOUT
  IF A$ = " " THEN GOSUB ZERODELTA   'reset max-min
  IF (A$ = "G") OR (A$ = "W") OR (A$ = "R") OR (A$ = "P") OR (A$ = "B") THEN
     GOSUB SENDADMODE  'G W R P & B commands require a warm start
  END IF
RETURN

'**** Get checksum from M202 & display it. If checksum  is bad, then   ****
'**** pause & flag an error.                                           ****
'**** USES    - CHECKSUM%                                              ****
'**** CHANGES - CHECKSUM%, PRINTSUM%                                   ****
'**** CALLS   - SENDPACKET, WAITCHAR, GETCHAR, PRINTERROR, CLEARCOMMAND****
GETCHKSUM:
  OUTB1% = &H87                         'send checksum token
  OUTB2% = 1
  GOSUB sendpacket
  NUMCHARS% = 2: GOSUB WAITCHAR         'wait for 2 char
  GOSUB GETCHAR
  IF OUTB1% <> INPBYTE% THEN E$ = " Checksum token wrong ": GOTO PRINTERROR
  PRINTSUM% = CHECKSUM%                 'PRINTSUM% is for display only
  GOSUB GETCHAR
  IF CHECKSUM% <> (INPBYTE% + INPBYTE%) MOD 256 THEN  'M201's checksum must
    PRINT "M201 checksum (HEX) "; HEX$(INPBYTE%),     'be added in before
    PRINT " BASIC checksum (HEX) "; HEX$(CHECKSUM%)   'comparison to host's
    E$ = " Checksum error": GOTO PRINTERROR
  END IF
  GOSUB CLEARCOMMAND
  PRINT "Checksum ok ";
  CHECKSUM% = 0                         'checksum command zeros running total
RETURN

'**** Get digital inputs from the M202                                 ****
'**** CHANGES - LASTDIGINP%                                            ****
'**** CALLS   - SENDPACKET, WAITCHAR, GETCHAR                          ****
GETDIGIN:
  OUTB1% = &H80                       'digital input token
  OUTB2% = &H4C                       'select digital inputs on 202
  GOSUB sendpacket
  NUMCHARS% = 2: GOSUB WAITCHAR       'wait for 2 char
  GOSUB GETCHAR                         'should be &H80
  IF OUTB1% <> INPBYTE% THEN E$ = " Digital In token wrong ": GOTO PRINTERROR
  GOSUB GETCHAR                       'INPBYTE% is data
  GOSUB CLEARCOMMAND
  PRINT "Digital input "; INPBYTE%;
  LASTDIGINP% = INPBYTE%
RETURN

'****  Toggle the bipolar mode flag                                    ****
'****  USES    - BIPOLAR%, UNIPOLAR%                                   ****
'****  CHANGES - BU%, DOSYSCAL%                                        ****
SELECTBP:
  IF BU% = BIPOLAR% THEN BU% = UNIPOLAR% ELSE BU% = BIPOLAR%
  DOSYSCAL% = TRUE
RETURN

'****  Enter sampling rate for the A/D converter, then calculate       ****
'****  the filter cutoff frequency value from the  sampling rate       ****
'****  USES    - RATE, NUMAVG, BAUD%, ADCLK                            ****
'****  CHANGES - DOSYSCAL%, RATE, MODEREGMID%, MODEREGLO%, WAITIME     ****
'****  CALLS   - CLEARCOMMAND                                          ****
SELECTRATE:                     'select the sampling rate for the A/D
  DOSYSCAL% = TRUE
  GOSUB CLEARCOMMAND
  LOCATE 25, 1
  PRINT USING "            Enter sample rate between ## and "; MNRATE;
  PRINT USING "#### Hz "; MAXRATE; : INPUT ; RATE
  GOSUB CLEARCOMMAND
  IF (RATE < MNRATE) OR (RATE > MAXRATE) THEN GOTO SELECTRATE
CALCRATE:
  MODECODE = ADCLK / RATE     'calculate the mode register data
  MODEREGLO% = MODECODE AND &HFF
  MODEREGMID% = (MODECODE AND &H700) / 256
  RATE = ADCLK / (((MODEREGMID% AND &H7) * 256) + MODEREGLO%)
CALCWAIT:
  WAITIME = (NUMAVG / RATE) + BAUD% / 10 + 2  'set default time to wait
RETURN

'****  Toggle the word length flag                                     ****
'****  USES    - BITS24%, BITS16%                                      ****
'****  CHANGES - WL%, WORDCNT%, MAXBITS%                               ****
SELECTWL:
  IF WL% = BITS24% THEN
    WL% = BITS16%: WORDCNT% = 2: MAXBITS% = 16
  ELSE
    WL% = BITS24%: WORDCNT% = 3: MAXBITS% = 24
  END IF
RETURN

'****  Select the next averaging factor for the A/D                    ****
'****  CHANGES - AVERAGE%, NUMAVG                                      ****
'****  CALLS   - SENDPACKET, CALCWAIT, CLEARCOMMAND                    ****
NEXTAVG:
  LOCATE 25, 1
  PRINT "Press + increase, - decrease averaging, Enter to continue ";
  PRINT USING "#####              "; NUMAVG;
GETAKEY:
  A$ = INKEY$
  IF A$ = "+" THEN AVERAGE% = (AVERAGE% + 1) MOD 16
  IF A$ = "-" THEN AVERAGE% = AVERAGE% - 1
  IF AVERAGE% < 0 THEN AVERAGE% = 15
  NUMAVG = 2 ^ AVERAGE%
  IF A$ <> CHR$(13) THEN GOTO NEXTAVG
SENDAVG:
  OUTB1% = 4                  'set average token
  OUTB2% = AVERAGE%: NUMAVG = 2 ^ AVERAGE%
  GOSUB sendpacket
  GOSUB CALCWAIT             'set default time to wait
  GOSUB CLEARCOMMAND
RETURN

'****  Prompt for channel control code, send data to the M202          ****
'****  CHANGES - CHANNEL%, EXTCCODE%                                   ****
'****  CALLS   - SENDPACKET, CLEARCOMMAND, SETCCODE, CLEARSDCOUNT      ****
CONTROLCODE:                            'send control code to M202
  GOSUB CLEARCOMMAND
  INPUT ; "Enter Channel"; CHANNEL%
  IF CHANNEL% < 0 OR CHANNEL% > 7 THEN GOTO CONTROLCODE
  GOSUB CLEARCOMMAND
SETCCODE:
  OUTB1% = &H1                         'control code command
  OUTB2% = (CHANNEL% * 16) + DACENABLE%   'M202 send DACENABLE% with ccode
  GOSUB sendpacket
  GOSUB CLEARSDCOUNT
RETURN

'****  Prompt for digital output code, send data to the M202           ****
'****  CHANGES - LASTDIGOUT%                                           ****
'****  CALLS   - SENDPACKET, CLEARCOMMAND                              ****
SETDIGOUT:                              'send digital output code to M202
  OUTB1% = 2                           'digital output token
  GOSUB CLEARCOMMAND
  INPUT ; "Enter Output Code"; OUTB2%
  GOSUB CLEARCOMMAND
  IF OUTB2% < 0 OR OUTB2% > 255 THEN GOTO SETDIGOUT
  LASTDIGOUT% = OUTB2%
  GOSUB sendpacket
RETURN

'****  Send warm start token to the M202, then send mode register data ****
'****  use after changing standby, rate, gain, uni/bipolar, or wordlen ****
'****  USES    - MODEREGHI%, MODEREGMID%, MODEREGLO%, INPBYTE%         ****
'****  CHANGES - NUMCHARS%                                             ****
'****  CALLS   - SENDPACKET, WAITXX, SETMODEREG, WAITCHAR, GETCHAR     ****
'****            READMODE                                              ****
SENDADMODE:
  OUTB1% = &H84                'warm start token
  GOSUB sendpacket
  NUMCHARS% = 1
  GOSUB WAITCHAR
  GOSUB SETMODEREG
  OUTB1% = MODEREGHI%: OUTB2% = MODEREGMID%
  GOSUB sendpacket            'send mode register data
  OUTB1% = MODEREGLO%: GOSUB sendpacket
  GOSUB GETCHAR     'Rev 2
  IF INPBYTE% <> &H84 THEN E$ = " wrong warm start token": GOTO PRINTERROR
  GOSUB READMODE               'check A/D control register & vars
RETURN

'****  Change scanning type                                            ****
'****  USES    - STYPE%, NEEDTOSIGNON%                                 ****
'****  CHANGES - NEEDTOSIGNON%, STYPE%, CC%(), NUMCCS%,SCANINTERVAL#   ****
'****  CALLS   - CALCSCANTIME, SCANTYPE, DOSCAN, SIGNON                ****
'****            EMPTYBUFFER, SETCCODE, FDELAY                         ****
SCANMENU:
  NEEDTOSIGNON% = FALSE             'flag sign-on not necessary
SCANMENU1:
  CLS : NUMCCS% = 0
  FOR N = 0 TO 5
  CC%(N) = CHAN%(N) MOD 16 - INT(CHAN%(N) / 16) + 1
  NUMCCS% = NUMCCS% + CC%(N)
  NEXT N
  LOCATE 1, 27: PRINT "SCANNING  CONFIGURATION"
  LOCATE 3, 14: PRINT "CHANNEL      EXTERNAL CONTROL CODE      NUMBER OF"
  LOCATE 4, 54: PRINT "DATA POINTS"
  IF STYPE% = 0 THEN
    LOCATE 7, 18: PRINT INT(SINGLECHAN% / 16);
    PRINT "               "; SINGLECHAN% MOD 16; "                 1"
    LOCATE 10, 15: PRINT "   PRESS 0 - 5 FOR CHANNEL"
  ELSE
    LOCATE 5, 29: PRINT "FROM           TO        THIS CHANNEL"
    FOR N = 0 TO 5
    LOCATE 7 + N * 2, 17
    PRINT USING "#)"; N; : PRINT , INT(CHAN%(N) / 16), CHAN%(N) MOD 16, CC%(N); ""
    NEXT N
  END IF
  LOCATE 19, 17
  PRINT USING "6) Scan Interval  ######.#### seconds"; SCANINTERVAL#;
  IF (GAIN% <> 1) AND (STYPE% = 2) THEN STYPE% = 0 'no cal scan unless gain=1
  LOCATE 21, 17: PRINT "7) Scan type: "; SCANTYPE$(STYPE%)
  LOCATE 23, 11: PRINT "  Enter) Perform the scan"
  LOCATE 25, 15: PRINT "Esc) To continue";
  IF NOT SINGLESCAN% THEN PRINT "       data points per scan"; NUMCCS%;
  A$ = INPUT$(1): VALA% = VAL(A$)
  IF (VALA% = 0) AND (A$ <> "0") THEN VALA% = 10  'flag invalid keystroke
  IF (VALA% >= 0) AND (VALA% <= 5) THEN GOSUB GETCHANNEL
  IF VALA% = 6 THEN GOSUB CALCSCANTIME
  IF VALA% = 7 THEN GOSUB SCANTYPE
  IF A$ = CHR$(13) THEN GOSUB DOSCAN: GOTO SCANMENU1
  IF A$ = CHR$(27) THEN
    POLLED% = TRUE
    IF NEEDTOSIGNON% THEN
      GOSUB SIGNON
    ELSE
       LOCATE 25, 10: PRINT "           Clearing serial buffer"; SPACE$(20);
       GOSUB EMPTYBUFFER
       DOSYSCAL% = FALSE
       GOSUB SETCCODE: GOSUB FDELAY
    END IF
    RETURN
  END IF
  IF VALA% > 7 THEN SOUND 15000, 1        'invalid keystroke
  GOTO SCANMENU1

'****  Change scanning type                                            ****
'****  USES    - GAIN%, STYPE%                                         ****
'****  CHANGES - STYPE%                                                ****
SCANTYPE:
  STYPE% = (STYPE% + 1) MOD 3
  IF (GAIN% <> 1) AND (STYPE% = 2) THEN STYPE% = 0 'no cal scan unless gain=1
EXITSCANTYPE:
RETURN

'*****************************************************************************
'Because all aspects of the scanning function of the Model 202 are
'programmable, it can be difficult to determine what the optimal settings are
'for any particular task. Also, if the A/D is not performing as intended, it
'can be hard to determine the cause. An overview of how to set up the 201
'from a programmer's perspective follows.
'
'The A/D is constantly performing conversions at DATARATE. This rate is set in
'the mode registers of the A/D chip itself. Slower DATARATEs provide lower
'noise, higher DATARATEs allow higher frequencies to be seen, and allow more
'data points to be taken in the same time period.
'
'For higher frequency applications the DATARATE should be set for the
'appropriate low-pass filter cutoff frequency. The cutoff frequency is
'approximately one quarter the DATARATE.
'
'For DC and very low frequency applications, you would normally select the
'lowest possible DATARATE. That lowest rate is determined by all the
'programmable scanning variables - scan type, the number of channels scanned,
'averaging, filter setting, word length and BAUD rate. Even though the
'DATARATE is actually sent to the A/D before the rest of the scanning details
'during the sign-on process, it should be calculated last. Details for Model
'202 follow:
'
'
'For Model 202:
'
'   BAUD%               = 0 for 9600, 1 for 4800, etc.
'   NUMCCS%             = number of channels to scan
'   NUMAVG              = number of conversions to average
'                         (1, 2, 4, 8, 16, 32 etc)
'   WORDCNT%            = the number of bytes in the conversion (2 or 3)
'
'Here, we have solved for the minimum sample rate given the scan interval:
'
'   MINSAMPRATE = (NUMAVG + 2) / ((SCANINTERVAL# - (WORDCNT% * .002333) *
'                  2 ^ BAUD%) / NUMCCS% - .03)
'
'A 5 percent margin should provide comfort. The following line would take
'care of that:
'
'   RATE =   1.05 * MINSAMPRATE
'*****************************************************************************

'****  Calculates Min/Max scanning rate, prompts for a new rate, then  ****
'****  calculates the # of rollovers for that rate                     ****
'****  USES    - GETSI%, NUMCCS%, BAUD%, RATE, WORDCNT%, NUMAVG        ****
'****            FTIME, STYPE%                                         ****
'****  CHANGES - GETSI%, MAXTIME#, MINTIME#, BAUDTIME, SCANINTERVAL#   ****
'****            SECROLL#, ROLLSCAN#, SCAN%, SCAN1%, SCAN2%, AVERAGE   ****
CHKSCANTIME:
  GETSI% = FALSE
CALCSCANTIME:
  CLS
  IF NUMCCS% = 0 THEN
    CLS
    PRINT "You must enter control codes first, press any key to continue"
    B$ = INPUT$(1): RETURN
  END IF
  MINTIME# = .007 * (2 ^ BAUD%) + (NUMCCS%) * (FTIME + (NUMAVG + 2) / RATE)
  BAUDTIME = .0011 * (2 ^ BAUD%) * (NUMCCS% * WORDCNT% + 2)
  IF STYPE% = 2 THEN                      'self-cal
    MINTIME# = MINTIME# + (.0042 * (2 ^ BAUD%)) + 2 * (FTIME + 4 / RATE)
    BAUDTIME = .0011 * (2 ^ BAUD%) * ((NUMCCS% + 2) * WORDCNT%)
  END IF
  IF STYPE% = 0 THEN                    'single channel
    MINTIME# = NUMAVG / RATE + (.0005 * (BAUD% + 1))
    BAUDTIME = .0011 * (2 ^ BAUD%) * WORDCNT%
  END IF

  MAXTIME# = 3386 * (2 ^ BAUD%) 'maximum scan interval (secs) @ 9600 baud
                                'adjusted for baud rate
  SECROLL# = 4882.8125# / (2 ^ BAUD%)   '# of rollovers for 1 second @ 9600 baud
                                        'adjusted for baud rate
  'If performing pass through commands while scanning, then
  'you may have to add as much as .1 seconds to the minimum
  'scan interval to avoid a scanning error.
  IF MINTIME# < BAUDTIME THEN MINTIME# = BAUDTIME  'baudrate limited scan
  IF MINTIME# > MAXTIME# THEN
    AVERAGE% = AVERAGE% - 1: GOSUB SENDAVG
    GETSI% = TRUE
    PRINT : PRINT "Reduced averaging to"; NUMAVG; "- Press any key to continue"
    B$ = INPUT$(1)
    GOTO CALCSCANTIME
  END IF
  IF BAUDRATE = MINTIME# THEN                   'if mintime# was set to baudtime
    LOCATE 15, 20: PRINT "minimum interval set by baud rate"
  END IF
  LOCATE 10, 10: PRINT "Enter scan interval ";
  MINTIME# = INT((MINTIME# + .00005) * SECROLL#) / SECROLL#
  PRINT USING "#####.####  - "; MINTIME#;
  MAXTIME# = INT(MAXTIME# * SECROLL#) / SECROLL#
  PRINT USING "#######.####  seconds"; MAXTIME#;
 
  IF GETSI% THEN INPUT ; SCANINTERVAL# ELSE GETSI% = TRUE
  ROLLSCAN# = SECROLL# * (SCANINTERVAL# + .00005) - 1     'rollovers per scan
  SCAN2% = INT(ROLLSCAN# / 65536)             'calculate scan interval vars
  SCAN2# = SCAN2%
  ROLLSCAN# = ROLLSCAN# - (SCAN2# * 65536)
  SCAN1% = INT(ROLLSCAN# / 256)                        'put in M201 format
  SCAN1# = SCAN1%
  ROLLSCAN# = ROLLSCAN# - (SCAN1# * 256)
  SCAN% = INT(ROLLSCAN#)
  SCANINTERVAL# = (SCAN% + SCAN1# * 256 + SCAN2# * 65536 + 1) / SECROLL#
   IF (SCANINTERVAL# > MAXTIME#) OR (SCANINTERVAL# < MINTIME#) THEN
    CLS
    LOCATE 15, 10: PRINT "Scan interval out of limits, press any key to continue"
    A$ = INPUT$(1)
    GOTO CALCSCANTIME
  END IF
  GETSI% = TRUE: A$ = ""
RETURN

'****  Get starting & ending control codes for a channel               ****
'****  USES    - STYPE%, VALA%, EXTERN%, CHAN%(N)                      ****
'****  CHANGES - SINGLECHAN%, CHAN%(N), STARTCC%, ENDCC%               ****
GETCHANNEL:
  IF STYPE% = 0 THEN
    CLS : LOCATE 12, 10
    PRINT "Channel number:    "; VALA%
    LOCATE 14, 10
    INPUT ; "Enter external control code, 0 - 15 "; EXTERN%
    SINGLECHAN% = VALA% * 16 + EXTERN% MOD 16
    GOTO EXITGETCHANNEL
  END IF
  CLS : LOCATE 23, 4
  PRINT "Enter an end code smaller than the start code to skip this channel"
  LOCATE 12, 1: PRINT "Channel number:    "; VALA%
  PRINT : INPUT "Enter starting control code"; STARTCC%
  IF (STARTCC% < 0) OR (STARTCC% > 15) THEN GOTO GETCHANNEL
GETEND:
  PRINT : INPUT "Enter ending control code"; ENDCC%
  IF (ENDCC% < 0) OR (ENDCC% > 15) THEN GOTO GETEND
  IF STARTCC% > ENDCC% THEN STARTCC% = 1: ENDCC% = 0
  CHAN%(VALA%) = STARTCC% * 16 + ENDCC%
EXITGETCHANNEL:
RETURN

'****  Write the setup variables to the file DRIVR202.CFG in the       ****
'****  default directory                                               ****
SAVECONFIG:
  ON ERROR GOTO WRITERR
  OPEN "DRIVR202.CFG" FOR OUTPUT AS #2
  WRITE #2, MODEREGHI%, MODEREGMID%, MODEREGLO%, CHANNEL%, FILTER%, AVERAGE%
  WRITE #2, DACENABLE%, BAUD%, COM$, INPBUF%, NUMAVG, BU%, GAIN%, RATE, WL%
  WRITE #2, CHAN%(0), CHAN%(1), CHAN%(2), CHAN%(3), CHAN%(4), CHAN%(5)
  WRITE #2, SINGLECHAN%
  WRITE #2, SCAN%, SCAN1%, SCAN2%, SCANINTERVAL#, STYPE%, RESERVE%
  CLOSE : ON ERROR GOTO 0
RETURN
WRITERR:
   CLS : PRINT "Unable to save DRIVR202.CFG file, press any key to continue"
   A$ = INPUT$(1)
RESUME 3000
3000 ON ERROR GOTO 0: RETURN

'****  Read the setup variables from the DRIVR202.CFG file in the      ****
'****  default directory                                               ****
'****  CALLS   - SETVARS, SETMODREG                                    ****
READCONFIG:
  GOSUB SETVARS
  ON ERROR GOTO READERR
  OPEN "DRIVR202.CFG" FOR INPUT AS #2
  INPUT #2, MODEREGHI%, MODEREGMID%, MODEREGLO%, CHANNEL%, FILTER%, AVERAGE%
  INPUT #2, DACENABLE%, BAUD%, COM$, INPBUF%, NUMAVG, BU%, GAIN%, RATE, WL%
  INPUT #2, CHAN%(0), CHAN%(1), CHAN%(2), CHAN%(3), CHAN%(4), CHAN%(5)
  INPUT #2, SINGLECHAN%
  INPUT #2, SCAN%, SCAN1%, SCAN2%, SCANINTERVAL#, STYPE%, RESERVE%
  CLOSE
4000
  ON ERROR GOTO 0
  IF WL% THEN WORDCNT% = 3: MAXBITS% = 24 ELSE WORDCNT% = 2: MAXBITS% = 16
  FTIME = .03                 'default filter delay time (secs)
  GOSUB SETMODEREG
RETURN
'****  Default values if configuration file cannot be located          ****
READERR:
  CLS : PRINT "Unable to read DRIVR202.CFG file, press any key to continue"
  A$ = INPUT$(1)
  COM$ = "COM1:": CHANNEL% = 7: INPBUF% = 512: GAIN% = 1
  NUMAVG = 1: RATE = 10: WL% = BITS24%: SCANINTERVAL# = 1
RESUME 4000

'****  Clears the command line                                         ****
CLEARCOMMAND:
  LOCATE 25, 1: PRINT SPACE$(80); : LOCATE 25, 57
RETURN

'****  Check serial communications with the M202                       ****
'****  USES    - INPBYTE%, OUTBYTE%                                    ****
'****  CHANGES - WAITIME, NUMCHARS%                                    ****
'****  CALLS   - WAITCHAR, GETCHAR                                     ****
ECHOTEST:
  OUTBYTE% = 1: i = 1: ERRCNT = 0: CLS
  LOCATE 10, 20: PRINT "BYTES SENT              ERRORS"
  LOCATE 12, 46: PRINT "0"
  LOCATE 25, 20: PRINT "Press any key to continue";
ECHOLOOP:
  PRINT #1, CHR$(OUTBYTE%);
  WAITIME = 1: NUMCHARS% = 1: GOSUB WAITCHAR      'wait for 1 char, 1 sec
  LOCATE 12, 28
  GOSUB GETCHAR
  LOCATE 12, 20
  PRINT USING "##########               "; i;
  IF INPBYTE% <> OUTBYTE% THEN BEEP: ERRCNT = ERRCNT + 1: PRINT ERRCNT
  i = i + 1                              'byte count
  OUTBYTE% = (OUTBYTE% + 1) MOD 255
  IF OUTBYTE% = 0 THEN OUTBYTE% = 1       'skip 0, it's the end echo token
  IF INKEY$ = "" THEN GOTO ECHOLOOP
RETURN

'****  Exit choices                                                    ****
'****  CALLS   - SAVECONFIG                                            ****
EXITPROGRAM:
      CLS : LOCATE 12, 20: PRINT "SAVE CONFIGURATION DATA ? (Y/N) "
      A$ = UCASE$(INPUT$(1))
      IF A$ = "Y" THEN GOSUB SAVECONFIG: GOTO EXITPROGRAM1
      IF A$ <> "N" THEN GOTO EXITPROGRAM
EXITPROGRAM1:
      CLS : LOCATE 12, 20: PRINT "EXIT PROGRAM ? (Y/N) "
      A$ = UCASE$(INPUT$(1))
      IF A$ = "N" THEN RETURN
      IF A$ <> "Y" THEN GOTO EXITPROGRAM1
      END    'leave the program

'****  Get a char from the serial port, update checksum                ****
'****  USES    - LOC(1)                                                ****
'****  CHANGES - CHECKSUM%                                             ****
GETCHAR:
  IF LOC(1) <> 0 THEN INPBYTE% = ASC(INPUT$(1, #1)) ELSE INPBYTE% = 0
  CHECKSUM% = (CHECKSUM% + INPBYTE%) MOD 256
RETURN

'****  Display M202 error handler                                      ****
'****  USES   - INPBYTE%, E$, LOC(1)                                   ****
'****  CHANGES - A$                                                    ****
PRINTERROR:
  BEEP
  PRINT : PRINT " error   "; E$
  PRINT " at "; TIME$; " on "; DATE$
  PRINT LOC(1); " byte(s) in input buffer"
  IF LOC(1) = 0 THEN GOTO PRINTLAST
  BYTES = LOC(1)
  FOR N = 1 TO BYTES
    INPBYTE% = ASC(INPUT$(1, #1))
    PRINT " byte "; N; "="; INPBYTE%; "( or, in hex) "; HEX$(INPBYTE%)
  NEXT N
  PRINT
PRINTLAST:
  PRINT " Last received byte may have been an error code."
  PRINT " The last byte was "; INPBYTE%; "  ";
  IF INPBYTE% = 0 THEN PRINT "Communication check error"
  IF INPBYTE% = 1 THEN PRINT "ChkSum error"
  IF INPBYTE% = 2 THEN PRINT "CmdPending error"
  IF INPBYTE% = 3 THEN PRINT "Master reset code"
  IF INPBYTE% = 4 THEN PRINT "Drdy error"
  IF INPBYTE% = 5 THEN PRINT "sign-on error"
  IF INPBYTE% = 6 THEN PRINT "Baud error"         '7 is unused
  IF INPBYTE% = 8 THEN PRINT "Pass through error"
  IF INPBYTE% = 9 THEN PRINT "Data request error"
  IF INPBYTE% = 10 THEN PRINT "Scan error"
  IF INPBYTE% = 11 THEN PRINT "A/D power up error"
ERRKEY:
  PRINT : PRINT " Press Q to restart. ";
  PRINT " (Your last-saved .CFG file will be read from the disk.)"
  A$ = UCASE$(INPUT$(1))
  IF A$ = "Q" THEN RUN ELSE GOTO ERRKEY

'****  Send master reset to M202                                       ****
SENDMRESET:
    OUTB1% = &H0           'master reset token
    PRINT #1, CHR$(OUTB1%);
RETURN

'****  Send two bytes to M202, & a checksum                            ****
'****  USES    -  OUTB1%, OUTB2%                                       ****
'****  CHANGES -  CHKSUM%                                              ****
sendpacket:
  PRINT #1, CHR$(OUTB1%);
  PRINT #1, CHR$(OUTB2%);
  CHKSUM% = ((OUTB1% + OUTB2%) AND 255)
  PRINT #1, CHR$(CHKSUM%);
RETURN

'****   Wait XX amount of time                                         ****
'****  USES    - TIMER, XX                                             ****
'****  CHANGES - XX                                                    ****
WAITXX:
  IF TIMER + XX > 86399 THEN GOTO WAITXX     'wait for end of day rollover
  XX = TIMER + XX
WAITLOOP:
  IF TIMER < XX THEN GOTO WAITLOOP
RETURN

'****  Wait for NUMCHARS% char's in serial buffer, or WAITIME seconds  ****
'****  USES    - TIMER, WAITTIME, NUMCHARS%                            ****
'****  CHANGES - CHECKKEY%, CANCELFLAG%                                ****
'****  CALLS   - CANCEL                                                ****
WAITCHARORKEY:
  CHECKKEY% = TRUE: GOTO WAITCHAR1
WAITCHAR:
  CHECKKEY% = FALSE
WAITCHAR1:
  CANCELFLAG% = FALSE
  IF TIMER + WAITIME > 86399 THEN GOTO WAITCHAR1  'wait for end of day
  XX = TIMER + WAITIME                            'time to wait
HOLDON:
  IF CHECKKEY% THEN
    IF TOGGLE$ = "*" THEN TOGGLE$ = " " ELSE TOGGLE$ = "*"
    PRINT TOGGLE$; : LOCATE , 1
    A$ = UCASE$(INKEY$)
    IF A$ <> "" THEN
      GOSUB CANCEL
      CANCELFLAG% = TRUE: RETURN
    END IF
  END IF
  IF (TIMER < XX) AND (LOC(1) < NUMCHARS%) THEN GOTO HOLDON
  IF (LOC(1) < NUMCHARS%) THEN
    CLS : PRINT "only "; LOC(1);
    PRINT " out of "; NUMCHARS%; " chars received"
    ' BASIC can lock-up after certain serial errors
    ' disabling and reenabling serial interrupts fixes the problem
    ' if your COM ports are at other addresses, change the next line
    IF COM$ = "COM1:" THEN PORTADDRESS = &H3F9 ELSE PORTADDRESS = &H2F9
    TEMP = INP(PORTADDRESS): OUT PORTADDRESS, 0: OUT PORTADDRESS, TEMP
    E$ = " Serial port time-out": GOTO PRINTERROR
  END IF
RETURN

'****  Send master reset token to the M202                             ****
'****  USES    -  INPBYTE%                                             ****
'****  CHANGES -  WAITIME                                              ****
'****  CALLS   -  SENDMRESET, WAITXX, GETCHAR, SET300BAUD, EMPTYBUFFER ****
'****             PRINTERROR                                           ****
DOMRESET:                               'send master reset command
  PRINT "         Reset           ";
  GOSUB SENDMRESET
  XX = .5: GOSUB WAITXX
  PRINT "Clearing serial buffer"; SPACE$(20);
  GOSUB EMPTYBUFFER
  IF INPBYTE% = 3 THEN GOTO EXITDOMRESET  'right return code
  E$ = " M202 did not respond correctly to Master Reset"
  GOTO PRINTERROR
EXITDOMRESET:
  GOSUB SET300BAUD             'successful reset sets M202 to 300 baud
RETURN

'****  Cancels current data request command                            ****
'****  USES    - BAUD%, INPBYTE%                                       ****
'****  CALLS   - WAITXX, EMPTYBUFFER                                   ****
CANCEL:
  OUTB1% = &H85
  PRINT #1, CHR$(OUTB1%);
  XX = (BAUD% / 20) + .1    'wait from .1 to .4 seconds
  GOSUB WAITXX
  GOSUB EMPTYBUFFER
  IF INPBYTE% <> &H85 THEN E$ = "no response to cancel": GOTO PRINTERROR
RETURN

'****  Perform a scan                                                  ****
'****  USES    - DOSYSCAL%, STYPE%, SINGLECHAN%                        ****
'****            WORDCNT%, LOC(1), TIMER, SCAN%, SCAN1%, SCAN2%        ****
'****            CHAN0%, CHAN1%, CHAN2%, CHAN3%, CHAN4%, CHAN5%        ****
'****  CHANGES - CHAN0%, CHAN1%, CHAN2%, CHAN3%, CHAN4%, CHAN5%        ****
'****            COMPLETESCANS, SOTOKEN%                               ****
'****  CALLS   - DOMRESET, CHKSCANTIME, SIGNON, SETMODEREG, SENDPACKET ****
'****            WAITXX, SYSCAL, FDELAY, WAITCHAR, GETCHAR, DISPLAYSCAN****
DOSCAN:
  IF NUMCCS% = 0 THEN RETURN            'invalid scan
  CLS
  POLLED% = FALSE
  GOSUB DOMRESET
  GOSUB CHKSCANTIME
  SOTOKEN% = &H88               'scanning requires long signon
  GOSUB SIGNON
  OUTB1% = SCAN%: OUTB2% = SCAN1%: GOSUB sendpacket    'send scanning set up
  OUTB1% = SCAN2%: OUTB2% = CHAN%(0): GOSUB sendpacket
  OUTB1% = CHAN%(1): OUTB2% = CHAN%(2): GOSUB sendpacket
  OUTB1% = CHAN%(3): OUTB2% = CHAN%(4): GOSUB sendpacket
  OUTB1% = CHAN%(5): OUTB2% = 0: GOSUB sendpacket
  XX = .2: GOSUB WAITXX
  'first figure number of words per scan for normal scan mode
  'the extra 2 bytes are the start and end of scan markers
  SCANWORDS = NUMCCS% * WORDCNT% + 2
  IF DOSYSCAL% AND (STYPE% <> 2) THEN GOSUB SYSCAL
  OUTB1% = &H89                          'normal scan command token
  IF STYPE% = 2 THEN
    SCANWORDS = SCANWORDS + (WORDCNT% * 2)      'add two conversion results
    OUTB1% = &H8C                               'calibrate scan token
  END IF
  IF STYPE% = 0 THEN                     'single-channel scans don't
    SCANWORDS = WORDCNT%                 'have start and end markers
    OUTB1% = &H1                         'control code command
    OUTB2% = SINGLECHAN%                 'set channel
    GOSUB sendpacket
    GOSUB FDELAY
    OUTB1% = &H8B                        'single-channel scan token
  END IF
  IF (SCANWORDS + 100) > INPBUF% THEN
    CLS : LOCATE 12, 17
    PRINT "Input buffer too small, press any key to continue"
    A$ = INPUT$(1): RETURN
  END IF
  GOSUB sendpacket
  NUMCHARS% = 1: GOSUB WAITCHAR
  SCANSTART = TIMER                       'the first scan begins now
  STARTTIME$ = TIME$
  STARTDAY$ = DATE$: DAYS = 0
  GOSUB GETCHAR                          'remove start scan token
  IF INPBYTE% <> OUTB1% THEN E$ = "Scan start failure": GOTO PRINTERROR
  RUNNING% = TRUE
  CLS : LOCATE 25, 10
  PRINT "Press any key to end scan or Esc to quit immediately."
  LOCATE 8, 10: PRINT "Scanning started at              "; STARTTIME$
  LOCATE 10, 10
  PRINT USING "Scanning interval, seconds #########.####"; SCANINTERVAL#
  LOCATE 12, 10
  PRINT USING "Bytes expected per scan       ######"; SCANWORDS
SCANSTATUS:
  IF STARTDAY$ <> DATE$ THEN DAYS = DAYS + 86400: STARTDAY$ = DATE$
  TEMP = LOC(1)
  IF RUNNING% THEN ELAPSED = TIMER + DAYS - SCANSTART ELSE TEMP = TEMP - 1      'subtract end scanning token
  COMPLETESCANS = TEMP / SCANWORDS
  LOCATE 14, 10
  PRINT USING "Elapsed time, seconds       ########"; ELAPSED
  LOCATE 16, 10
  PRINT USING "Scans expected             #########"; INT(ELAPSED / SCANINTERVAL#) + 1
  LOCATE 18, 10
  PRINT USING "Scans completed            #########.##"; COMPLETESCANS
  IF NOT RUNNING% THEN
    IF COMPLETESCANS = INT(COMPLETESCANS) THEN
      TEMP = LOC(1)
      XX = WAITIME: GOSUB WAITXX
      IF LOC(1) = TEMP THEN GOTO EXITDOSCAN
     END IF
  END IF
  A$ = INKEY$         'get a keystroke
  IF LOC(1) > (INPBUF% - 100) THEN A$ = "XX": GOTO CUTSHORT
                             'issue cancel to prevent a buffer full error
  IF A$ = "" THEN GOTO SCANSTATUS          'no keystroke, buffer not full
  IF RUNNING% THEN                              'if keystroke, then
    OUTB1% = &H8A                               'send end scan token
    GOSUB sendpacket
    XX = .4: GOSUB WAITXX
    LOCATE 25, 10
    PRINT "Scanning will stop. Press ESC to truncate scan.       ";
    RUNNING% = FALSE
  END IF
  IF A$ <> CHR$(27) THEN GOTO SCANSTATUS    'wait for end of scan
CUTSHORT:                                   'unless Esc pressed
  OUTB2% = &H85           'send cancel
  PRINT #1, CHR$(OUTB2%);
  XX = .2: GOSUB WAITXX
  IF A$ = "XX" THEN
    LOCATE 20, 10
    PRINT "Buffer full. Press space to continue";
    A$ = INPUT$(1)
  END IF
EXITDOSCAN:
  GOSUB DISPLAYSCAN
RETURN


'****  Display scan data                                              ****
'****  USES    - COMPLETESCANS, SCANWORDS, INPBYTE%,                   ****
'****            STYPE%, WORDCNT%, CHAN%(N)                            ****
'****  CHANGES - SCANS, NUMCHARS%, NEEDTOSIGNON%, DOSYSCAL%            ****
'****  CALLS   - GETCHAR, VOLTSCALE, VOLTDISPLAY, WAITCHAR, GCONV      ****
'****            GETCHKSUM                                             ****
DISPLAYSCAN:
  IF INKEY$ <> "" THEN GOTO DISPLAYSCAN    'clear the keyboard buffer
  CLS
  PRINT " CC     chan 0       chan 1       chan 2      ";    'display header
  PRINT " chan 3       chan 4       chan 5": PRINT
  FOR i = 0 TO 15: PRINT i: NEXT
  SCAN = 1
  NUMCHARS% = WORDCNT%

SCANLOOP:
  IF SCAN > INT(COMPLETESCANS) THEN GOTO SCANSDONE
  IF STYPE% = 0 THEN         'single-channel scan
    CC = (SINGLECHAN% MOD 16): N = INT(SINGLECHAN% / 16)
    GOSUB CONVDISP
    GOTO ONEDONE
  END IF
  GOSUB GETCHAR                            'get start of scan token
  IF INPBYTE% <> &HF0 THEN E$ = "Scan framing failure": GOTO PRINTERROR
  IF STYPE% = 2 THEN            'calibration scan, get extra words
    GOSUB WAITCHAR: GOSUB GCONV
    LOCATE 23, 1: PRINT "Offset cal "; : GOSUB VOLTSCALE: GOSUB VOLTDISPLAY
    GOSUB WAITCHAR: GOSUB GCONV
    LOCATE 23, 25: PRINT "Full scale cal "; : GOSUB VOLTSCALE
    GOSUB VOLTDISPLAY
  END IF
  FOR N = 0 TO 5              'get # of datapoints for each channel
  STARTCC = INT(CHAN%(N) / 16): ENDCC = CC%(N) + STARTCC - 1
    FOR CC = STARTCC TO ENDCC       'display all data points for each channel
    GOSUB CONVDISP
    NEXT CC
  NEXT N
  PRINT
  GOSUB GETCHAR    'get end of scan token &H0F
  IF INPBYTE% <> &HF THEN E$ = "Scan framing failure": GOTO PRINTERROR
ONEDONE:
  LOCATE 25, 1: PRINT USING "SCAN ######  of"; SCAN;
  PRINT INT(COMPLETESCANS);
  SCAN = SCAN + 1
  PRINT "  Press any key to continue, ESC to quit          ";
  A$ = INPUT$(1)
  IF A$ = CHR$(27) THEN RETURN ELSE GOTO SCANLOOP

CONVDISP:
  GOSUB GCONV
  LOCATE (CC + 3), (N * 13) + 4
  GOSUB VOLTSCALE: GOSUB VOLTDISPLAY
RETURN

SCANSDONE:
  CLS
  NEEDTOSIGNON% = FALSE
  IF (COMPLETESCANS = 0) OR (COMPLETESCANS <> INT(COMPLETESCANS)) THEN
    LOCATE 10, 10: PRINT "Last scan was incomplete, press any key to continue"
    A$ = INPUT$(1): NEEDTOSIGNON% = TRUE
  ELSE
    GOSUB GETCHAR    'get end of scanning token &H8A
    IF INPBYTE% <> &H8A THEN
      PRINT "No end scan token, press any key to continue"
      A$ = INPUT$(1): NEEDTOSIGNON% = TRUE
    END IF
  END IF
  GOSUB EMPTYBUFFER   'it is possible that another scan is in the buffer
  IF NEEDTOSIGNON% THEN DOSYSCAL% = TRUE:  ELSE GOSUB GETCHKSUM
RETURN

'      Additional programmer's  information
'
'****                      Command tokens  (hexidecimal)              ****
'         80 = get digital input word
'              OUTB2% = &H0C      expansion select RD
'              OUTB2% = &H2C      expansion select WR
'              OUTB2% = &H4C      202 digital input
'              OUTB2% = &H8C      expansion select ENABLE 4
'              OUTB2% = &HCC      expansion select ENABLE 6
'         81 = get a conversion result
'              OUTB2% = 0         202 A/D converter
'         82 = do an offset calibration
'         83 = do a fullscale calibration
'         84 = set A/D mode registers
'         85 = cancel
'         86 = version
'         87 = get checksum
'         88 = put in sleep mode (also sign-on token)
'         89 = start a scan
'         8A = end a scan
'         8B = single-channel scan
'         8C = scan with calibration
'         0  = master reset and end echo
'         1  = select A/D channel
'         2  = set external optically isolated output port
'         3  = change filter settings
'         4  = change averaging
'         5  = reserved
'         6  = Read 8253 counter
'         7  = Write 8253 counter
'         8  = aux output
'         9  = aux output
'

'****************** M202 ***********************
'****** all Model 202-specific routines ******
'****************** M202 ***********************
' The lower 4 bits in the CC register (Command token 1) control both DACs
' Bit 3= enable outputs, Bit 2= enables counter itself
' Bits 1 & 0 select the counter number
' First select a counter with a token 1 command, then use
' command token 7 to write a count to the seleted counter
' Note that DAC A maximum voltage is 1 count less than 5 volts
' and DAC B minimum voltage is 1 count above 0 volts

SETM202VARS:

TIMEBASECNT = 10000             ' full-scale dac count 2 to 65535
COUNTSPERVOLT = TIMEBASECNT / 5 ' counts for 1 volt
DACENABLE% = 0                  ' DAC's disabled, 8 = enabled
                                ' there are three 16-bit counters
                                ' Counter 0 is used to trigger counters 1 & 2 fi
  TIMEBASE% = 0                 ' Counter 0 MODE 2, 10 MHz clock input
                                ' any count from 2 to 65535 can be used
                                ' higher resolution means slower settling
                             
  ANALOGOUTA% = 1               ' Counter 1 analog output
                                ' MODE 1, 10 MHz clock, gated by counter 0
                                ' TIMEBASECNT to 1 counts = 0 to 5 VDC out
 
  ANALOGOUTB% = 2               ' Counter 2 analog output
                                ' MODE 1, 10 MHz clock, gated by counter 0
                                ' 1 to TIMEBASECNT counts = 0 to 5 VDC out

RETURN                          'end of SETM202VARS

INITCOUNTERS:                                       ' initialize counters
        CNTPORT% = TIMEBASE%                        ' set counter 0
        CNTRCOUNT = TIMEBASECNT: GOSUB INITCNTR     ' load count for timebase
        CNTPORT% = ANALOGOUTA%                      ' set counter 1
        CNTRCOUNT = TIMEBASECNT: GOSUB INITCNTR     ' load count for 0 VDC
        CNTPORT% = ANALOGOUTB%                      ' set counter 2
        CNTRCOUNT = 1: GOSUB INITCNTR               ' load count for 0 VDC
        DACAVOLTS = 0: DACBVOLTS = 5 / TIMEBASECNT
RETURN

INITCNTR:    ' expects CNTPORT%      counter  0, 1 or 2;
              '   and CNTRCOUNT       value of count
 OUTB1% = 1                                     ' Output control code token
 OUTB2% = DACENABLE% OR (CHANNEL% * 16) OR &H87 ' select mode register
 GOSUB sendpacket
 OUTB1% = 7                                     ' write counter token
 IF CNTPORT% = 0 THEN CMODE% = 4 ELSE CMODE% = 2'if counter 0 then mode 2
 OUTB2% = 48 + CNTPORT% * 64 + CMODE%           ' set counter mode
 GOSUB sendpacket
WRITECNTR:
 CH = INT(CNTRCOUNT / 256)                      ' get hi and lo bytes of count
 CL = FIX(CNTRCOUNT - CH * 256)
 OUTB1% = 1                                     ' cc token
 OUTB2% = CNTPORT% OR 4 OR DACENABLE% OR (CHANNEL% * 16) OR &H80
 GOSUB sendpacket                               ' select counter number
 OUTB1% = 7                                     ' write lo count
 OUTB2% = CL
 GOSUB sendpacket
 OUTB1% = 7                                     ' write hi count
 OUTB2% = CH
 GOSUB sendpacket
 CMODE% = 1
RETURN

ANALOGOUT:                                      ' set analog output voltage
   CLS
   LOCATE 10, 30: PRINT "DAC A          DAC B";
   LOCATE 12, 30: PRINT USING "#.####         "; DACAVOLTS; DACBVOLTS
   LOCATE 25, 15: PRINT "1 for DAC A, 2 for DAC B, ESC to return";
   PRINT "    Resolution  1 /"; TIMEBASECNT;
GETDAC:
   CNT$ = UCASE$(INPUT$(1))
   IF CNT$ = "" THEN GOTO GETDAC
   IF CNT$ = CHR$(27) THEN GOTO EXITANALOGOUT
   IF CNT$ <> "1" AND CNT$ <> "2" THEN GOTO GETDAC
ENTERV:
   CLS : LOCATE 10, 20: INPUT ; "Enter output voltage 0 - 5V ", VOLTAGE
   IF VOLTAGE < 0 OR VOLTAGE > 5 THEN GOTO ENTERV
   CNTRCOUNT = (VOLTAGE * COUNTSPERVOLT)                ' calculate counts
   'the dac A is inverted with respect to dac B
   IF CNT$ = "1" THEN CNTRCOUNT = TIMEBASECNT - CNTRCOUNT
   'don't exceed the max count
   IF (CNTRCOUNT > TIMEBASECNT) THEN CNTRCOUNT = TIMEBASECNT
   IF CNTRCOUNT < 1 THEN CNTRCOUNT = 1                  ' 0 is illegal count
    IF CNT$ = "1" THEN
      CNTPORT% = ANALOGOUTA%                            ' set counter address
      DACAVOLTS = (TIMEBASECNT - CNTRCOUNT) / COUNTSPERVOLT' set last voltage
   ELSE
      CNTPORT% = ANALOGOUTB%                            ' set counter address
      DACBVOLTS = CNTRCOUNT / COUNTSPERVOLT             ' set last voltage
      END IF
   GOSUB WRITECNTR                                      ' set the voltage
   GOTO ANALOGOUT
EXITANALOGOUT:
CLS : RETURN

'**** Toggle DAC enable                                                ****
'**** CHANGES - DACENABLE%                                             ****
TOGGLEDACS:
  IF DACENABLE% = 8 THEN DACENABLE% = 0 ELSE DACENABLE% = 8
  GOSUB SETCCODE                                       ' send data to M202
RETURN

'**************** M202 *******************

